/*
 * as5040.h
 *
 *  Created on: 22.04.2011
 *      Author: Wolfram Koerver
 */

#ifndef AS5040_H
#define AS5040_H

#include <util/delay.h>
#include <avr/io.h>

#define AS5040_CLK		SBIT( PORTA, 0)
#define AS5040_DDR_CLK	SBIT( DDRA, 0)

#define AS5040_DI		SBIT( PORTA, 1)
#define AS5040_DDR_DI	SBIT( DDRA, 1)		// input from AS5040 <---

#define AS5040_PRG		SBIT( PORTA, 2)
#define AS5040_DDR_PRG	SBIT( DDRA, 2)		// output to  AS5040 --->

#define AS5040_CS		SBIT( PORTA, 3)
#define AS5040_DDR_CS	SBIT( DDRA, 3)

#define AS5040_PORT		PINA
#define AS5040_PIN_DI	PINA1

/**
 *  @name AS5040 Functions
 */

extern void as5040_init(void);

/**
 @brief  This routine initializes AS5040 Port Pins of ATMEGA 644.3 pins are needed at minimum to read position.
 If programming of offset position is required the AS5040 prog pin has to be connected as well.
 CS 	= chip select
 DI 	= data in
 CLK 	= Clock pulse
 Connections for this program to an ATMEGA 644:
 AS5040 :
 GND      Pin 7  <----> GND
 Prog     Pin 8  <----> PORTA PINA.2 (Optional)
 Dout     Pin 9  <----> PORTA PINA.1
 CLK      Pin 10 <----> PORTA PINA.0
 CS       Pin 11 <----> PORTA PINA.3
 +Ub = 5V Pin 16 <---->


 @param    none
 @param    none
 @return   none
*/

extern uint16_t as5040_ssi(void);
/**
 @brief    This routine reads the synchronous serial input (SSI) position data from AS5040 into a 16 bit data variable resolution is 10 bits. Maximum readout rate is approx. 10khz. Recommended 1kHz.
 @param    none
 @return   16bit uint_16 variable : position and status data of AS5040
*/

extern void as5040_zero(uint16_t zpos);
/**
 @brief    This routine programs the OTP default register of the AS5040. Be careful, the programmed value should be programmed only once at the beginning, when encoder is in physical 0 position.
 Read current value and write this into the zero position register will resets the position values to 0 offset.
 Every programming just overwrites the zero register, independent of the real axis position.
 @param    zpos : uint16_t zero position 0...1024 (see data shet of AS5040)
 @return   none
*/

extern uint16_t reverse_u16 (uint16_t in);
/**
 @brief    This routine reverses the 16 bits of an uint16 value
 @param    in : uint16_t input value
 @return   return : uint16_t reversed output value
*/


// define bits as bit field to access them in program just by referencing bit = 1 etc.
struct bits
{
	unsigned char b0:1 ;
	unsigned char b1:1 ;
	unsigned char b2:1 ;
	unsigned char b3:1 ;
	unsigned char b4:1 ;
	unsigned char b5:1 ;
	unsigned char b6:1 ;
	unsigned char b7:1 ;
} __attribute__((__packed__));

#define SBIT_(port,pin)  ((*(volatile struct bits*)&port).b##pin)
#define	SBIT(x,y)	SBIT_(x,y)

#endif /* AS5040_H */
